// MainClass.java created on 21 November 2005 at 12:05

//=============================================================================
//
//    Copyright (C) 2005 Kevin Ashelford
//    Contact details:
//    Email: ashelford@cardiff.ac.uk
//    Address:  Cardiff School of Bioscience, Cardiff University, PO. Box 915,
//    Cardiff, UK. CF10 3TL
//
//    This program is free software; you can redistribute it and/or modify
//    it under the terms of the GNU General Public License as published by
//    the Free Software Foundation; either version 2 of the License, or
//    any later version.
//
//    This program is distributed in the hope that it will be useful,
//    but WITHOUT ANY WARRANTY; without even the implied warranty of
//    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
//    GNU General Public License for more details.
//
//    You should have received a copy of the GNU General Public License
//    along with this program; if not, write to the Free Software
//    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
//
//=============================================================================

package kea.programs.squirrel;

import java.lang.*;
import javax.swing.*;
import java.io.*;
import java.awt.event.*;
import java.awt.*;
import javax.swing.border.*;
import java.util.*;

import javax.help.*;
import java.net.*;

import kea.programs.frames.*;
import kea.sequences.nucleicAcid.*;
import kea.programs.dialogBoxes.io.*;
import kea.io.*;
import kea.event.*;
import kea.database.*;
import kea.exceptions.*;
import kea.programs.icons.KeaIcon;
import kea.programs.table.sequence.SequenceCollectionTableFactory;
import kea.programs.table.sequence.Column;
import kea.programs.table.sequence.SequenceCollectionTable;
import kea.sequences.nucleicAcid.OrientationChecker;
import kea.programs.text.*;
import kea.programs.text.SequenceComponentFactory;
import kea.programs.text.SequenceComponent;

////////////////////////////////////////////////////////////////////////////////
////////////////////////////////////////////////////////////////////////////////

/**
 
 @author  Kevin Ashelford
 @version 1.0
 */
public class MainClass extends SequenceCollectionFrame {
	
	//##########################################################################
	
	// CLASS FIELDS
	
	//======================================================================
	// Identify location of helpset - for help menu item.
	private static final String HELPSET_LOCATION = 
		MainClass.class.getResource("help/HelpSet.hs").toString();
	
	private static HelpSet _helpSet = null;
	private static HelpBroker _helpBroker = null;
	private static ActionListener _helpActionListener = null;
	
	static {
		URL url = null;
		try {
			url = new URL((new File(".")).toURL(), HELPSET_LOCATION);
			} 
		catch(MalformedURLException mue) {
			System.out.println(mue.getMessage());
			}
		
		
		try {
			_helpSet = new HelpSet(null, url);
			
			_helpBroker = _helpSet.createHelpBroker();
			_helpActionListener = new CSH.DisplayHelpFromSource(_helpBroker);
			} 
		catch(HelpSetException hse) {
			JOptionPane.showMessageDialog(
				null,
				"<html>Could not find the help files for this application.<br><br>" +
				"The program will still run correctly, but help will be unavailable.</html>",
				"HelpSet Exception", 
				JOptionPane.ERROR_MESSAGE
				);
			}
		
		}

	//======================================================================
	
	private static final String PROGRAM_NAME = "OrientationChecker";
	private static final String AUTHOR_NAME = "Kevin Ashelford";
	private static final String PROGRAM_DESCRIPTION = 
		"a program for identifying the orientation of 16S rRNA gene sequences.";	
	private static final String PROGRAM_VERSION = "1.0";
	
	private final ListObject SENSE = 
		new ListObject(
			"Sense sequences", 
			KeaIcon.Sense.getImageIcon()
			);
	private final ListObject ANTISENSE = 
		new ListObject(
			"Antisense sequences", 
			KeaIcon.Antisense.getImageIcon()
			);
	private final ListObject UNKNOWN = 
		new ListObject(
			"Sequences with unknown orientation", 
			KeaIcon.RedQuestionMark.getImageIcon()
			);
	private final ListObject SELECTED = 
		new ListObject(
			"Selected sequences", 
			KeaIcon.Ok.getImageIcon()
			);
	
	////////////////////////////////////////////////////////////////////////////
	
	// MEMBER FIELDS
	
	private final Action _toSenseAction = new ToSenseAction();
	private final Action _toAntisenseAction = new ToAntisenseAction();
	private final Action _runAction = new RunAction();
	private final Action _saveAction = new SaveAction();
	private final JLabel _senseLabel = new JLabel();
	private final JLabel _antisenseLabel = new JLabel();
	private final JLabel _unknownLabel = new JLabel();
	private final FastaFileFilter _fastaFileFilter = new FastaFileFilter();
	private final SequenceComponent _sequenceComponent = SequenceComponentFactory.create();
	private final SummaryPanel _summaryPanel = new SummaryPanel();
	private final JComboBox _comboBox1 = new JComboBox(
		new Object[]{
			SENSE,
			ANTISENSE,
			UNKNOWN,
			SELECTED
			}	
		);
	private final JComboBox _comboBox2 = new JComboBox(
		new Object[]{
			SENSE,
			ANTISENSE,
			UNKNOWN,
			SELECTED
			}	
		);
	private final SavePanel _savePanel = new SavePanel();
	private final SequencePanel _sequencePanel = new SequencePanel();
	
	private int _senseCounter = 0;
	private int _antisenseCounter = 0;
	private int _unknownCounter = 0;	
	
	//##########################################################################
	
	// ENTRY POINT
	
	/**
	 Entry point for program.
	 @param args the command line arguments
	 */
	public static void main(String[] args) {
		
		// Let user know that program does not accept command line arguments.
		if (args.length > 0) {
			System.err.println(
				"Program does not currently accept command line arguments."
				);
			}
		
		new MainClass();
		
		} // End of main.
	
	//##########################################################################
	
	// CONSTRUCTOR

	public MainClass() {			
		super(
			PROGRAM_NAME, 
			AUTHOR_NAME, 
			PROGRAM_DESCRIPTION, 
			PROGRAM_VERSION, 
			_helpActionListener,
			//new Dimension(900, 768),
			new Column[]{
				Column.Number, 
				Column.Code, 
				Column.Thumbnail,
				Column.Size, 
				Column.Orientation
				}
			);
		initialise();
		setVisible(true);
		
		} // End of constructor.
	
	//##########################################################################
	
	// METHODS
	
	private void initialise() {
			
		// Disable.
		_runAction.setEnabled(false);
		_saveAction.setEnabled(false);
		_comboBox1.setEnabled(false);
		_comboBox2.setEnabled(false);
		_summaryPanel.setEnabled(false);
		_savePanel.setEnabled(false);
		_sequencePanel.setEnabled(false);
		getOptionsMenu().setEnabled(false);
		
		// Ensure both comboboxes show same selected item.
		ActionListener l = new MyActionListener();
		_comboBox1.addActionListener(l);
		_comboBox2.addActionListener(l);
		
		_comboBox1.setRenderer(new SaveListCellRenderer());
		_comboBox2.setRenderer(new SaveListCellRenderer());
			
		// Ensure combo box does not fill all of toolbar.
		JPanel cbp = new JPanel(new BorderLayout());
		cbp.add(_comboBox2, BorderLayout.WEST);
		
		// Modify tool bar.
		getToolBar().add(_runAction);
		getToolBar().add(_toSenseAction);
		getToolBar().add(_toAntisenseAction);
		getToolBar().addSeparator();
		getToolBar().add(_saveAction);
		getToolBar().add(cbp);
		getToolBar().addSeparator();
		
		JPanel panel = this.getMainPanel();
		panel.setLayout(new BorderLayout());
		
		// North - new panel.
		JPanel upperPanel = new JPanel(new BorderLayout());
		panel.add(upperPanel, BorderLayout.NORTH);
		
			// North - Button panel.
			upperPanel.add(initialiseButtonPanel(), BorderLayout.NORTH);
		
			// Center -- summary panel.
			upperPanel.add(_summaryPanel, BorderLayout.CENTER);
		
		// Center - sequence component.
		panel.add(_sequencePanel, BorderLayout.CENTER);
		
		getSequenceCollectionTable().
			addSingleClickSequenceSelectedListener(
				new SingleClickSequenceSelectedListener()
				);
		
		addOpenFileListener(new MyOpenFileListener());
		addCloseFileListener(new MyCloseFileListener());
		
		} // End of method.
	
	////////////////////////////////////////////////////////////////////////////
	
	private JPanel initialiseButtonPanel() {
		
		JPanel panel = new JPanel(new BorderLayout());
		panel.setBorder(new EmptyBorder(5, 5, 5, 5));
		
		JPanel panel1 = new JPanel(new GridLayout(1, 1));
		JButton runButton = new JButton(_runAction);
		runButton.setPreferredSize(new Dimension(100, 70));
		panel1.add(runButton);
		panel1.setBorder(new EmptyBorder(5, 5, 5, 5));
		panel.add(panel1, BorderLayout.NORTH);
		
		JPanel panel2 = new JPanel(new GridLayout(1, 2, 5, 5));
		panel2.setBorder(new EmptyBorder(5, 5, 5, 5));
		
		JButton toSenseButton = new JButton(_toSenseAction);
		toSenseButton.setPreferredSize(new Dimension(100, 50));
		
		JButton toAntisenseButton = new JButton(_toAntisenseAction);
		toAntisenseButton.setPreferredSize(new Dimension(100, 50));
		
		panel2.add(toSenseButton);			
		panel2.add(toAntisenseButton);
		panel.add(panel2, BorderLayout.CENTER);
		
		panel.add(_savePanel, BorderLayout.SOUTH);
		
		return panel;
		
		} // End of method.
	
	////////////////////////////////////////////////////////////////////////////
	
	public void updateDisplay() {
		_senseLabel.setText(String.valueOf(_senseCounter));
		_antisenseLabel.setText(String.valueOf(_antisenseCounter));
		_unknownLabel.setText(String.valueOf(_unknownCounter));
		super.updateDisplay();
		}
	
	////////////////////////////////////////////////////////////////////////////
	
	private void createSaveDialog(
		String dialogTitle, 
		ListObject listObject, 
		Sequence[] sequences
		) {
		
		// Abort if no sequences to save.
		if (sequences.length == 0) {
			
			String s = null;
			if (listObject == SENSE) s = "There are no sense sequences to save!";
			else if (listObject == ANTISENSE) s = "There are no antisense sequences to save!";
			else if (listObject == UNKNOWN) s = "There are no sequences with an unknown orientation to save!";
			else if (listObject == SELECTED) s = "No sequences have been selected!"; 
			else throw new RuntimeException("Unknown list object: " + listObject);
			
			JOptionPane.showMessageDialog(
				this, 
				s, 
				"No sequences", 
				JOptionPane.INFORMATION_MESSAGE
				);
			return;
			}
		
		JFileChooser fc = new JFileChooser(){
			// Override method.
			public void approveSelection() {
				File file = getFileWithExtension(super.getSelectedFile());
				File directory = super.getCurrentDirectory();
				if (okToSave(file, directory)) {
					super.approveSelection();
					super.setSelectedFile(file);
					}
				else {return;}
				}
			};
			
		// Remove AcceptAll file filter.
		fc.removeChoosableFileFilter(fc.getAcceptAllFileFilter());
			
		// Add fasta file filter.
		fc.setFileFilter(_fastaFileFilter);
		
		// Default selected file.
		fc.setSelectedFile(null);
		
		// Give dialog box a title.
		fc.setDialogTitle(dialogTitle);
		
		// Save to file if valid filename generated by user.
		if (fc.showSaveDialog(this) == JFileChooser.APPROVE_OPTION) {
			File filename = fc.getSelectedFile();
			
			// SAVE TO FILE.
			try {
				PrintWriter pw = new PrintWriter(new FileWriter(filename));
				for (int i = 0; i < sequences.length; i++) {
					pw.println(">" + sequences[i].getCode());
					pw.println(sequences[i].getSequence());
					pw.println();
					}
				pw.close();
				} 
			catch (IOException ioe) {
				JOptionPane.showMessageDialog(
					this, 
					"Problem encountered whilst trying to save file: " + 
					ioe.getMessage(), 
					"Output Exception", 
					JOptionPane.ERROR_MESSAGE
					);
				}
			}
		
		} // End of method.
	
	////////////////////////////////////////////////////////////////////////////
	
	// Ensures correct extension is added (if not already done so by user).
	private File getFileWithExtension(File in) {
	
		// Take no action if file extension is already present.
		if (_fastaFileFilter.accept(in)) return in;
		
		// Create new file object with correct extension.
		return 
			new File(in.getAbsoluteFile() + "." + _fastaFileFilter.getExtension());
		
		} // End of method.
	
	////////////////////////////////////////////////////////////////////////////
	
	// Checks that no file is being overwritten.
	private boolean okToSave(File file, File directory) {
		
		// Get list of files in current directory.
		File[] files = directory.listFiles(_fastaFileFilter);
		
		// Check user supplied name with these.
		for (int i = 0; i < files.length; i++) {
			
			String name1 = files[i].getName();
			String name2 = file.getName();
			
			if (name1.equals(name2)) {
				return okToOverwrite(name1);
				}
			
			}
		
		// No existing file with same name therefore ok to save.
		return true;
		
		} // End of method.
	
	////////////////////////////////////////////////////////////////////////////
	

	// User confirms via dialog box that it is ok (or not) to overwrite 
	// existing file.
	private boolean okToOverwrite(String filename) {
		
		if (JOptionPane.showConfirmDialog(
			this,
			"'" + filename + "' already exists.  Do you want to replace it?",
			"Warning",
			JOptionPane.YES_NO_OPTION,
			JOptionPane.INFORMATION_MESSAGE
			) == JOptionPane.YES_OPTION) {return true;} else {return false;}
		
		} // End of method.
	
	//##########################################################################
	
	// INNER CLASSES
	
	private class SequencePanel extends JPanel {
		
		private TitledBorder titledBorder = new TitledBorder("View selected sequence");
		
		public SequencePanel() {
			setLayout(new BorderLayout());
			setBorder(
				new CompoundBorder(
					new EmptyBorder(5, 5, 5, 5),
					titledBorder
					)
				);
			_sequenceComponent.setEditable(false);
			add(new JScrollPane(_sequenceComponent), BorderLayout.CENTER);
		
			}
		
		public void setEnabled(boolean b) {
			if (b) titledBorder.setTitleColor(Color.BLACK);
			else titledBorder.setTitleColor(Color.GRAY);
			super.setEnabled(b);
			}
		
		} // End of inner class.
	
	//++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
	
	private class SavePanel extends JPanel {
	
		private TitledBorder titledBorder = new TitledBorder("Save");
		
		public SavePanel() {
			setLayout(new BorderLayout());
			setBorder(
				new CompoundBorder(
					new EmptyBorder(5, 5, 5, 5),
					titledBorder
					)
				);
			JPanel buttonPanel = new JPanel(new GridLayout(1,1));
			buttonPanel.setBorder(new EmptyBorder(0, 5, 5, 5));
			
			JButton saveButton = new JButton(_saveAction);
			saveButton.setPreferredSize(new Dimension(150, 35));
			
			buttonPanel.add(saveButton);
			add(buttonPanel, BorderLayout.WEST);

			JPanel comboPanel = new JPanel(new GridLayout(1,1));
			comboPanel.setBorder(new EmptyBorder(0, 5, 5, 5));
			comboPanel.add(_comboBox1);
			add(comboPanel, BorderLayout.CENTER);
			}
		
		public void setEnabled(boolean b) {
			if (b) titledBorder.setTitleColor(Color.BLACK);
			else titledBorder.setTitleColor(Color.GRAY);
			super.setEnabled(b);
			}
		
		} // End of inner class.
	
	//++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
	
	private class SummaryPanel extends JPanel {
		
		private JLabel heading1 = new JLabel("Sense sequences: ", KeaIcon.Sense.getImageIcon(), JLabel.LEFT);
		private JLabel heading2 = new JLabel("Antisense sequences: ", KeaIcon.Antisense.getImageIcon(), JLabel.LEFT);
		private JLabel heading3 = new JLabel("Unknown sequences: ", KeaIcon.RedQuestionMark.getImageIcon(), JLabel.LEFT);
		
		private TitledBorder titledBorder = new TitledBorder("Summary");
		
		public SummaryPanel() {
			setLayout(new GridLayout(3, 2, 5, 5));
			setBorder(
				new CompoundBorder(
					new EmptyBorder(5, 5, 5, 5),
					titledBorder
					)
				);

			add(heading1); add(_senseLabel); 
			add(heading2); add(_antisenseLabel);
			add(heading3); add(_unknownLabel);

			_toSenseAction.setEnabled(false);
			_toAntisenseAction.setEnabled(false);

			}
		
		public void setEnabled(boolean b) {
			heading1.setEnabled(b);
			heading2.setEnabled(b);
			heading3.setEnabled(b);
			if (b) titledBorder.setTitleColor(Color.BLACK);
			else titledBorder.setTitleColor(Color.GRAY);
			super.setEnabled(b);
			}
		
		} // End of inner class.
	
	//++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
	
	private class SaveListCellRenderer extends JLabel implements ListCellRenderer {
		
		public Component getListCellRendererComponent(
			JList list,
		  Object value,
		  int index,
		  boolean isSelected,
		  boolean cellHasFocus
		) {
			// Get current list object.
			ListObject listObject = (ListObject)value;
			
			// Set text of JLabel object.
			setText(listObject.getText());
			
			// Decide on icon to show.
			setIcon(listObject.getIcon());
			
			// Ensure icon looks disabled when combo box is disabled.
			if (_comboBox1.isEnabled()) setEnabled(true);
			else setEnabled(false);
			
			return this;
			
			}
		
		} // End of inner class.
	
	//++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
	
	private class ListObject {
		private final String text;
		private final ImageIcon icon;
		public ListObject(String t, ImageIcon i) {text = t;icon = i;}
		public String getText() {return text;}
		public ImageIcon getIcon() {return icon;}
		public String toString() {return text;}
		}
	
	//++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
	
	private class MyActionListener implements ActionListener {
		
		public void actionPerformed(ActionEvent ae) {
			
			JComboBox combo = (JComboBox)ae.getSource();
			Object item = combo.getSelectedItem();
			
			if (combo == _comboBox1) _comboBox2.setSelectedItem(item);
			if (combo == _comboBox2) _comboBox1.setSelectedItem(item);
			}
		
		} // End of inner class.
	
	//++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
	
	private class ToSenseAction extends AbstractAction {
		
		public ToSenseAction() {
			putValue(Action.NAME, "Convert all antisense to sense");
			putValue(Action.SMALL_ICON, KeaIcon.Sense.getImageIcon());
			putValue(Action.SHORT_DESCRIPTION, "Any antisense sequences are converted to their sense counterpart.");
			}
		
		public void actionPerformed(ActionEvent ae) {	
			
			// Do nothing if no sequences to change.
			if (_antisenseCounter == 0) return;
			
			// Run.
			new ToSenseThread();
			
			}
		
		} // End of inner class.
	
	//++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
	
	private class ToSenseThread extends Thread {
		
		public ToSenseThread() {
			setPriority(MIN_PRIORITY);
			start();
			}
		
		public void run() {
			// get current sequence collection.
			SequenceCollection oldCollection = getSequenceCollection();
			
			// Shouldn't happen, but just in case.
			if (oldCollection == null) return;
			
			getProgressComponent().getProgressBar().setIndeterminate(true);
			
			// Create new sequence collection.
			SequenceCollection newCollection = null;
			try {newCollection = convert(oldCollection);} 
			// Shouldn't happen but just in case.
			catch (Exception e) {throw new RuntimeException(e.getMessage());}
			
			// Update counters.
			_senseCounter = _antisenseCounter + _senseCounter;
			_antisenseCounter = 0;
			
			// Register new collection with display and update.
			setSequenceCollection(newCollection); 
			SwingUtilities.invokeLater(new Runnable(){public void run(){
				updateDisplay();
				}});
			
			
			// Clear sequence component (if necessary).
			_sequenceComponent.clearDisplay();
			
			// Disable button, enable other.
			_toAntisenseAction.setEnabled(true);
			_toSenseAction.setEnabled(false);
			
			getProgressComponent().getProgressBar().setIndeterminate(false);
			}
		
		// Creates a new sequence collection, converting sequences where necessary.
		private SequenceCollection convert(SequenceCollection oldCollection) 
			throws Exception {
			
			SequenceCollection newCollection = 
				SequenceCollectionFactory.createSequenceCollection();
			
			Sequence s = null;
			for (int i = 0; i < oldCollection.getSize(); i++) {	
				s = oldCollection.getSequence(i);
				if (s.getOrientation() == Orientation.ANTISENSE) {
					
					int position = s.getFivePrimePosition();
					
					s = SequenceFactory.createSequence(
						SequenceConvertor.createReverseComplement(s.getSequence()), 
						s.getName(), 
						s.getCode(), 
						Orientation.SENSE, 
						s.getStatus(), 
						s.getSummary()
						);
					s.setFivePrimePosition(position);
					
					}
				newCollection.addSequence(s);
				}
			return newCollection;
			}
		
		} // End of inner class.
	
	//++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
	
	private class ToAntisenseAction extends AbstractAction {
		
		public ToAntisenseAction() {
			putValue(Action.NAME, "Convert all sense to antisense");
			putValue(Action.SMALL_ICON, KeaIcon.Antisense.getImageIcon());
			putValue(Action.SHORT_DESCRIPTION, "Any sense sequences are converted to their antisense counterpart.");
			}
		
		public void actionPerformed(ActionEvent ae) {	
			
			// Do nothing if no sequences to change.
			if (_senseCounter == 0) return;
			
			// Run.
			new ToAntisenseThread();
			
			}
		
		} // End of inner class.
	
	//++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
	
	private class ToAntisenseThread extends Thread {
		
		public ToAntisenseThread() {
			setPriority(MIN_PRIORITY);
			start();
			}
		
		public void run() {
			
			// get current sequence collection.
			SequenceCollection oldCollection = getSequenceCollection();
			
			// Shouldn't happen, but just in case.
			if (oldCollection == null) return;
			
			getProgressComponent().getProgressBar().setIndeterminate(true);
			
			// Create new sequence collection.
			SequenceCollection newCollection = null;
			try {newCollection = convert(oldCollection);}  
			// Shouldn't happen but just in case.
			catch (Exception e) {throw new RuntimeException(e.getMessage());}
			
			// Update counters.
			_antisenseCounter = _antisenseCounter + _senseCounter;
			_senseCounter = 0;
			
			// Register new collection with display and update.
			setSequenceCollection(newCollection); 
			SwingUtilities.invokeLater(new Runnable(){public void run(){
				updateDisplay();
				}});
			
			// Clear sequence component (if necessary).
			_sequenceComponent.clearDisplay();
			
			// Disable button, enable other.
			_toAntisenseAction.setEnabled(false);
			_toSenseAction.setEnabled(true);
			
			getProgressComponent().getProgressBar().setIndeterminate(false);
			}
		
		// Creates a new sequence collection, converting sequences where necessary.
		private SequenceCollection convert(SequenceCollection oldCollection) 
			throws Exception {
			
			SequenceCollection newCollection = 
				SequenceCollectionFactory.createSequenceCollection();
			
			Sequence s = null;
			for (int i = 0; i < oldCollection.getSize(); i++) {	
				s = oldCollection.getSequence(i);
				if (s.getOrientation() == Orientation.SENSE) {
					
					int position = s.getFivePrimePosition();
					
					s = SequenceFactory.createSequence(
						SequenceConvertor.createReverseComplement(s.getSequence()), 
						s.getName(), 
						s.getCode(), 
						Orientation.ANTISENSE, 
						s.getStatus(), 
						s.getSummary()
						);
					s.setFivePrimePosition(position);
					}
				newCollection.addSequence(s);
				}
			return newCollection;
			}
		
		} // End of inner class.
	
	//++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
	
	private class SaveAction extends AbstractAction {
		
		public SaveAction(){
			putValue(Action.NAME, "Save to file: ");
			putValue(Action.SMALL_ICON, KeaIcon.Save.getImageIcon());
			}
		
		public void actionPerformed(ActionEvent ae) {
			
			// Identify what sequences are to be saved.
			ListObject value = (ListObject)_comboBox1.getSelectedItem();
			
			// Get relevant sequences.
			
			Sequence[] sequences = null;
			
			if (value == SENSE) {
				sequences = getSequences(Orientation.SENSE);
				createSaveDialog("Save sense sequences to file", value, sequences);
				} 
			else if (value == ANTISENSE) {
				sequences = getSequences(Orientation.ANTISENSE);
				createSaveDialog("Save antisense sequences to file", value, sequences);
				}
			else if (value == UNKNOWN) {
				sequences = getSequences(Orientation.UNKNOWN);
				createSaveDialog("Save sequences with unknown orientation to file", value, sequences);
				}
			else if (value == SELECTED) {
				sequences = getSequenceCollectionTable().getSelectedSequences();
				createSaveDialog("Save selected sequences to file", value, sequences);
				}
			else {
				throw new RuntimeException("'" + value + "' not supported.");
				}
			
			}
		
		private Sequence[] getSequences(Orientation orientation) {
			
			SequenceCollection sc = getSequenceCollection();
			ArrayList array = new ArrayList(); 
			for (int i = 0; i < sc.getSize(); i++) {
				if (orientation == sc.getSequence(i).getOrientation()) {
					array.add(sc.getSequence(i));
					}
				}
			
			Sequence[] sequences = new Sequence[array.size()];
			array.toArray(sequences);
			return sequences;
			}
		
		} // End of inner class.
	
	//++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
	
	private class RunThread extends Thread {
		
		public RunThread() {
			setPriority(Thread.MIN_PRIORITY);
			start();
			}
		
		public void run() {
		
			getProgressComponent().getProgressBar().setIndeterminate(true);

			OrientationChecker oc = new OrientationChecker(); 
			
			// reset counters.
			_senseCounter = 0;
			_antisenseCounter = 0;
			_unknownCounter = 0;

			// Get sequences.
			SequenceCollection sequences = getSequenceCollection();

			// for each sequence check orientation and position.
			Sequence sequence = null;
			Orientation orientation = null;
			for (int i = 0; i < sequences.getSize(); i++) {	
				sequence = sequences.getSequence(i);
				orientation = oc.getOrientation(sequence); 
				
				sequence.setFivePrimePosition(oc.getPosition(sequence));
				
				sequence.setOrientation(orientation);
				if (orientation == Orientation.SENSE) _senseCounter++;
				else if (orientation == Orientation.ANTISENSE) _antisenseCounter++;
				else _unknownCounter++; 
				}

			// update table and summary.
			updateDisplay();

			// Enable buttons.
			_toSenseAction.setEnabled(true);
			_toAntisenseAction.setEnabled(true);

			// Disable run button.
			_runAction.setEnabled(false);

			getProgressComponent().getProgressBar().setIndeterminate(false);

			// Enable / disable buttons according to number of sense/antisense.
			if (_senseCounter == sequences.getSize() - _unknownCounter) {
				_toAntisenseAction.setEnabled(true);
				_toSenseAction.setEnabled(false);
				}
			if (_antisenseCounter == sequences.getSize() - _unknownCounter) {
				_toAntisenseAction.setEnabled(false);
				_toSenseAction.setEnabled(true);
				}

			// Enable.
			_saveAction.setEnabled(true);
			_comboBox1.setEnabled(true);
			_comboBox2.setEnabled(true);
			_summaryPanel.setEnabled(true);
			_savePanel.setEnabled(true);
			_sequencePanel.setEnabled(true);
			}
		
		} // End of inner class.
	
	//++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
	
	private class MyOpenFileListener implements OpenFileListener {
		
		public void openFileAction() {
			
			// Clear all displays.
			_sequenceComponent.clearDisplay();
			_senseLabel.setText("");
			_antisenseLabel.setText("");
			_unknownLabel.setText("");
			
			// Reset counters to zero.
			_senseCounter = 0;
			_antisenseCounter = 0;
			_unknownCounter = 0;
			 
			// Enable run button.
			_runAction.setEnabled(true);
			
			// Disable buttons.
			_toSenseAction.setEnabled(false);
			_toAntisenseAction.setEnabled(false);
			_summaryPanel.setEnabled(false);
			_savePanel.setEnabled(false);
			_sequencePanel.setEnabled(false);
		
			}
		
		} // End of inner class.
	
	//++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
	
	private class MyCloseFileListener implements CloseFileListener {
		
		public void closeFileAction() {
			
			// Clear all displays.
			_sequenceComponent.clearDisplay();
			_senseLabel.setText("");
			_antisenseLabel.setText("");
			 _unknownLabel.setText("");
			
			// Disable run button.
			_runAction.setEnabled(false);
			
			// Disable buttons.
			_toSenseAction.setEnabled(false);
			_toAntisenseAction.setEnabled(false);
			
			// Reset counters to zero.
			_senseCounter = 0;
			_antisenseCounter = 0;
			_unknownCounter = 0;
			
			// Disable.
			_saveAction.setEnabled(false);
			_comboBox1.setEnabled(false);
			_comboBox2.setEnabled(false);
			_summaryPanel.setEnabled(false);
			_savePanel.setEnabled(false);
			_sequencePanel.setEnabled(false);
			
			}
		
		} // End of inner class.
	
	//++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
	
	private class RunAction extends AbstractAction {
		
		public RunAction() {
			putValue(Action.NAME, "Check orientation");	
			putValue(Action.SMALL_ICON, KeaIcon.InvestigateSequence.getImageIcon());
			putValue(Action.SHORT_DESCRIPTION, "Identifies whether uploaded sequences have a sense or antisense orientation.");
			}
		
		public void actionPerformed(ActionEvent ae) {
			new RunThread();
			}
		
		} // End of inner class.
	
	//++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
	
	private class SingleClickSequenceSelectedListener implements SequenceSelectedListener {
		
		public void sequenceSelected(SequenceSelectionEvent sse) {
			
			Sequence sequence = sse.getSelectedSequence();
			_sequenceComponent.setSequence(sequence);
			_sequenceComponent.updateDisplay();
			}
		
		} // End of inner class. 
	
	//##########################################################################
	
	} // End of class.

////////////////////////////////////////////////////////////////////////////////
////////////////////////////////////////////////////////////////////////////////
